SystemOrganization addCategory: #MultiplayerDungeonGame!

Morph subclass: #Game
	instanceVariableNames: 'connectionSocket'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MultiplayerDungeonGame'!

!Game methodsFor: 'as yet unclassified' stamp: 'HenrySolberg 7/15/2014 04:28'!
debugPrint: aString
	Transcript show: self asString, ':  ' , aString , '(' , Time now asMilliSeconds asString , ')' , Character cr asString! !

!Game methodsFor: 'as yet unclassified' stamp: 'WestonPanther 7/18/2014 15:33'!
handleCommand: receivedData withBoard: aBoard andPlayer: aPlayer 
	(receivedData = '28') ifTrue: [aBoard moveGameObjectLeft: aPlayer].
	(receivedData = '31') ifTrue: [aBoard moveGameObjectDown: aPlayer].
	(receivedData = '29') ifTrue: [aBoard moveGameObjectRight: aPlayer].
	(receivedData = '30') ifTrue: [aBoard moveGameObjectUp: aPlayer].
	(receivedData = '13') ifTrue: [
		((aBoard boxAt: (aPlayer x) at: (aPlayer y)) usable class = Stairs) ifTrue: [ 
			self stairsUsedBy: aPlayer withBoard: aBoard.
		] ifFalse: [ 
			aPlayer useOnBoard: aBoard 
		] 	
	]! !

!Game methodsFor: 'as yet unclassified' stamp: 'WestonPanther 7/17/2014 15:54'!
stairsUsedBy: aPlayer withBoard: aBoard
	self subclassResponsibility ! !

Game subclass: #GameClient
	instanceVariableNames: 'windowPosition isRunning isRunningLock gb resyncing resyncingMessage'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MultiplayerDungeonGame'!

!GameClient methodsFor: 'as yet unclassified' stamp: 'HenrySolberg 7/18/2014 02:53'!
handleKeystroke: anEvent 
	| keyValue | 
	resyncing ifFalse: 
	[  
 		keyValue := anEvent keyValue.
		Transcript show: keyValue.
		self handleCommand: (keyValue asString) withBoard:  gb  andPlayer: (gb player).
		isRunning ifTrue: [ connectionSocket sendData: keyValue asString ].
	]
	ifTrue: 
	[ 
		resyncingMessage isNil ifFalse: [ resyncingMessage delete. ].
		resyncingMessage := GameObject imagePath: 'resyncing.png'. 
		resyncingMessage center: self center. 
		self addMorph: resyncingMessage. 
	]! !

!GameClient methodsFor: 'as yet unclassified' stamp: 'WestonPanther 7/18/2014 17:35'!
handleSocket: interactionSocket withWindow: window
	| receivedData |
		
		[ [interactionSocket isConnected ] whileTrue:  [  
			receivedData := ByteString new.
			[receivedData endsWith: 'Complete>' ] whileFalse: [receivedData := receivedData , interactionSocket receiveData.].
			(receivedData endsWith: '<boardComplete>') ifTrue: [  self syncWithWindow:  window usingData:  receivedData ].
			(receivedData endsWith: '<hashComplete>') ifTrue: [ 
				(gb hash asString = (receivedData readStream upToAll: '<hashComplete>')) ifFalse: [ 
					resyncing := true.
					interactionSocket sendData: '<resync>'.
				]
			].
			(window isInWorld) ifFalse: [ ConnectionTimedOut signal ]
			]
		] on: ConnectionTimedOut
		do: [].! !

!GameClient methodsFor: 'as yet unclassified' stamp: 'HEnrySolberg 7/11/2014 05:12'!
handlesMouseOver: anEvent 
   ^ true! !

!GameClient methodsFor: 'as yet unclassified' stamp: 'HenrySolberg 7/18/2014 02:36'!
initialize
	super initialize.
	isRunningLock := Mutex new.
	self isRunning: false.
	windowPosition := 50@50.
	gb := GameBoard new.
	resyncing := false.! !

!GameClient methodsFor: 'as yet unclassified' stamp: 'WestonPanther 7/18/2014 17:36'!
interactUsing: interactionSocket
	| window |
		
	[ 
		window := StandardWindow new.
		window open.
		self color: Color transparent.
		self handleSocket: interactionSocket withWindow: window
	] ensure: [
		interactionSocket closeAndDestroy.
		self delete. 
	] ! !

!GameClient methodsFor: 'as yet unclassified' stamp: 'HEnrySolberg 7/10/2014 08:39'!
isRunning
	^ isRunningLock critical: [ isRunning ]! !

!GameClient methodsFor: 'as yet unclassified' stamp: 'HEnrySolberg 7/10/2014 08:39'!
isRunning: aBoolean
	isRunningLock critical: [ isRunning := aBoolean ]! !

!GameClient methodsFor: 'as yet unclassified' stamp: 'HEnrySolberg 7/10/2014 08:44'!
joinIP: anAddress.
	isRunningLock critical: [
	self isRunning ifTrue: [ ^ self ].
	self isRunning: true].
	connectionSocket := Socket newTCP.
	connectionSocket connectTo: anAddress port: 9999.
	[self interactUsing: connectionSocket] fork! !

!GameClient methodsFor: 'as yet unclassified' stamp: 'HEnrySolberg 7/10/2014 09:40'!
mouseEnter: anEvent 
    anEvent hand newKeyboardFocus: self! !

!GameClient methodsFor: 'as yet unclassified' stamp: 'HEnrySolberg 7/11/2014 05:16'!
mouseLeave: anEvent 
	anEvent hand newKeyboardFocus: nil! !

!GameClient methodsFor: 'as yet unclassified' stamp: 'WestonPanther 7/17/2014 16:00'!
stairsUsedBy: aPlayer withBoard: aBoard
	resyncing := true.
	resyncingMessage := (GameObject imagePath: 'resyncing.png').
	resyncingMessage center: self center.
	self addMorph: (resyncingMessage)
	
	! !

!GameClient methodsFor: 'as yet unclassified' stamp: 'HEnrySolberg 7/10/2014 08:39'!
stop
	self isRunning: false! !

!GameClient methodsFor: 'as yet unclassified' stamp: 'HenrySolberg 7/18/2014 02:51'!
syncWithWindow: aWindow usingData: receivedData

	(receivedData size = 0) ifFalse:
	[
		receivedData readStream upToAll: '<boardComplete>'.
		gb := (FLMaterializer materializeFromByteArray:  receivedData asByteArray).
		(gb class = GameBoard) ifFalse: [self error: 'Non-GameBoard data received!!'].

		windowPosition := aWindow position.
		aWindow addMorph: gb.
"		aWindow position: windowPosition."
		aWindow height: gb height + 30.
		aWindow width: gb width + 10.
		self position: (aWindow layoutBounds topLeft).
		self height: (aWindow layoutBounds bottom - aWindow layoutBounds top).
		self width: (aWindow layoutBounds right - aWindow layoutBounds left).
		aWindow addMorph: self.
		resyncing := false.
		resyncingMessage isNil ifFalse: [ resyncingMessage delete].
	]
! !

Game subclass: #GameServer
	instanceVariableNames: 'isRunning isRunningLock numConnections'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MultiplayerDungeonGame'!

!GameServer commentStamp: '<historical>' prior: 0!
A GameServer is opens a socket for connections and carries out the game logic. It keeps track of all connections, and their current game state.

Instance Variables
	connectionSocket:		<Object>
	isRunning:		<Object>
	isRunningLock:		<Object>
	numConnections:		<Object>

connectionSocket
	- xxxxx

isRunning
	- xxxxx

isRunningLock
	- xxxxx

numConnections
	- xxxxx
!

!GameServer methodsFor: 'as yet unclassified' stamp: 'WestonPanther 7/18/2014 17:27'!
handleSocket: interactionSocket withBoard: gb withPlayer: player
	|receivedData|
	
		[interactionSocket isConnected ] whileTrue:  
		[
			[
				receivedData := interactionSocket receiveData .
				(receivedData = '<resync>') ifFalse: [  
					self handleCommand: receivedData withBoard: gb andPlayer: player.
					interactionSocket sendData: (gb hash) asString, '<hashComplete>'.
				] ifTrue: [ 
		 			self sendSerializedBoard: gb onSocket: interactionSocket.
				]
			]  on: ConnectionTimedOut
				do: [].
		]. ! !

!GameServer methodsFor: 'as yet unclassified' stamp: 'WestonPanther 7/9/2014 13:14'!
initialize
	super initialize.
	isRunningLock := Mutex new.
	self isRunning: false.
	numConnections := 0
	! !

!GameServer methodsFor: 'as yet unclassified' stamp: 'WestonPanther 7/8/2014 22:56'!
interactOnConnection
	| interactionSocket |
	interactionSocket := connectionSocket waitForAcceptFor: 1 ifTimedOut: [^self].
	[self interactUsing: interactionSocket] fork! !

!GameServer methodsFor: 'as yet unclassified' stamp: 'WestonPanther 7/18/2014 17:27'!
interactUsing: interactionSocket
	| player  gb |
		
	[ 
		numConnections := numConnections + 1.
		player := Denizen new form: (ImageReadWriter formFromFileNamed: 'player.png') .
		(gb := self setupGameBoard: player) openInWindow.		
		self sendSerializedBoard: gb onSocket: interactionSocket.

		self handleSocket: interactionSocket withBoard: gb withPlayer: player.

	] ensure: [
	interactionSocket closeAndDestroy ]! !

!GameServer methodsFor: 'as yet unclassified' stamp: 'WestonPanther 7/8/2014 22:52'!
isRunning
	^ isRunningLock critical: [ isRunning ]! !

!GameServer methodsFor: 'as yet unclassified' stamp: 'WestonPanther 7/8/2014 22:53'!
isRunning: aBoolean
	isRunningLock critical: [ isRunning := aBoolean ]! !

!GameServer methodsFor: 'as yet unclassified' stamp: 'WestonPanther 7/9/2014 13:16'!
numConnections
	^numConnections ! !

!GameServer methodsFor: 'as yet unclassified' stamp: 'WestonPanther 7/9/2014 13:14'!
numConnections: aNum
	numConnections := aNum! !

!GameServer methodsFor: 'as yet unclassified' stamp: 'WestonPanther 7/18/2014 15:33'!
sendSerializedBoard: aBoard onSocket: aSocket
	|serializedBoard|
	
	serializedBoard := (FLSerializer serializeToByteArray: aBoard).
	serializedBoard := ByteString fromByteArray: serializedBoard.
	aSocket sendData: serializedBoard , '<boardComplete>'.! !

!GameServer methodsFor: 'as yet unclassified' stamp: 'WestonPanther 7/8/2014 22:58'!
serve
	[ [ self isRunning ]
	whileTrue: [ self interactOnConnection ] ]
	ensure: [ connectionSocket closeAndDestroy ]! !

!GameServer methodsFor: 'as yet unclassified' stamp: 'WestonPanther 7/17/2014 16:02'!
setupGameBoard: aPlayer
	|gb|
	gb := (GameBoard boardSize: 15 windowSize: 450).
	gb player: aPlayer.
	^gb at: 1 at: 1 addGameObject: (aPlayer)! !

!GameServer methodsFor: 'as yet unclassified' stamp: 'WestonPanther 7/17/2014 15:58'!
stairsUsedBy: aPlayer withBoard: aBoard
	aPlayer useOnBoard: aBoard! !

!GameServer methodsFor: 'as yet unclassified' stamp: 'WestonPanther 7/8/2014 22:57'!
start
	isRunningLock critical: [
	self isRunning ifTrue: [ ^ self ].
	self isRunning: true].
	connectionSocket := Socket newTCP.
	connectionSocket listenOn: 9999 backlogSize: 10.
	[ self serve ] fork! !

!GameServer methodsFor: 'as yet unclassified' stamp: 'WestonPanther 7/8/2014 22:55'!
stop
	self isRunning: false! !

Morph subclass: #GameBoard
	instanceVariableNames: 'boardSize windowSize board lineWidth player'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MultiplayerDungeonGame'!

!GameBoard commentStamp: '<historical>' prior: 0!
A GameBoard is a matrix of game boxes and methods for interactions on the game board.

Instance Variables
	board:		matrix of game boxes
	boardSize:		int representing width and height (X by X)
	lineWidth:		width of borders between moxes (int)
	player:		the controllable game object
	windowSize:		square pixel size (int)

!

!GameBoard class methodsFor: 'as yet unclassified' stamp: 'HEnrySolberg 7/10/2014 03:54'!
boardSize: aBoardSizeInt windowSize: aWindowSizeInt
	^self new boardSize: aBoardSizeInt windowSize: aWindowSizeInt.! !

!GameBoard methodsFor: 'accessing' stamp: 'HenrySolberg 7/18/2014 12:38'!
at: anX at: aY
	"access the denizen at a coord"
	^(board at: anX at: aY) occupant.! !

!GameBoard methodsFor: 'modifying' stamp: 'WestonPanther 7/16/2014 16:20'!
at: anX at: aY addGameObject: aGameObject
	(aGameObject isUsable) ifTrue: [ (board at: anX at: aY) setUsable: aGameObject ]
							ifFalse: [ (board at: anX at: aY) setOccupant: aGameObject ].
	aGameObject x: anX; y: aY.
	aGameObject position: (self positionOfX: anX Y: aY).
	aGameObject resize: (windowSize / boardSize) @ (windowSize / boardSize).
	self addMorph: aGameObject.! !

!GameBoard methodsFor: 'accessing' stamp: 'WestonPanther 7/16/2014 17:34'!
boardSize
	^boardSize! !

!GameBoard methodsFor: 'initializing' stamp: 'HenrySolberg 7/18/2014 04:10'!
boardSize: aBoardSizeInt windowSize: aWindowSizeInt
	self delete.
	self initialize.
	boardSize := aBoardSizeInt.
	windowSize := aWindowSizeInt.
	lineWidth := 4.
	board := Matrix new: aBoardSizeInt.
	
	"self error."
	self buildMaze.
	self at: boardSize at: boardSize addGameObject: (Stairs imagePath: 'stairs.png')! !

!GameBoard methodsFor: 'initializing' stamp: 'HenrySolberg 7/18/2014 04:16'!
boardSize: aBoardSizeInt windowSize: aWindowSizeInt keepWindow: window
	self delete.
	self initialize.
	window addMorph: self.
	boardSize := aBoardSizeInt.
	windowSize := aWindowSizeInt.
	lineWidth := 4.
	board := Matrix new: aBoardSizeInt.
	
	"self error."
	self buildMaze.
	self at: boardSize at: boardSize addGameObject: (Stairs imagePath: 'stairs.png')! !

!GameBoard methodsFor: 'accessing' stamp: 'HenrySolberg 7/16/2014 01:11'!
boxAt: anX at: aY
	"returns a fully walled box if out of bounds"
	(anX > boardSize | (aY > boardSize) | (anX < 1) | (aY < 1)) ifTrue: [^GameBox new].
	^(board at: anX at: aY).! !

!GameBoard methodsFor: 'mazing' stamp: 'HenrySolberg 7/17/2014 08:54'!
buildMaze
	|spacing  boxMatrix setMatrix wallList |
	
	spacing := windowSize/boardSize.
	setMatrix := Matrix new: boardSize.
	boxMatrix := self makeBoxMatrixSize: boardSize withSpacing: spacing.
	wallList := OrderedCollection new.
	
	setMatrix indicesDo: [ :i :j |  
		setMatrix at: i at:j put: (Set with: (boxMatrix at:  i at: j)).
	].

	boxMatrix do: [ :box |
		wallList add: { box. true }.
		wallList add: { box. false }
	].
	wallList shuffle.
	
	boxMatrix := (self mazify: boxMatrix withSet:  setMatrix withWalls: wallList).
	
	boxMatrix do: [ :box | self putBox: box spacing: spacing ]! !

!GameBoard methodsFor: 'modifying' stamp: 'HEnrySolberg 7/10/2014 05:26'!
fromX: fromX fromY: fromY toX: toX toY: toY
	|mover|
	mover := (board at: fromX at: fromY) occupant.
	mover x: toX; y: toY.
	mover position: (self positionOfX: toX Y: toY).
	(board at: toX at: toY) setOccupant: mover.
	(board at: fromX at: fromY) setOccupant: nil.! !

!GameBoard methodsFor: 'hashing' stamp: 'HenrySolberg 7/15/2014 03:56'!
hash
	^board hash ! !

!GameBoard methodsFor: 'mazing' stamp: 'WestonPanther 7/16/2014 15:42'!
makeBoxMatrixSize: size withSpacing: spacing
	|boxMatrix newBox|
	
	boxMatrix := Matrix new: size.
	
	boxMatrix indicesDo: [ :i :j | 
		newBox := GameBox northWall: false eastWall: true southWall: true westWall: false tileImage: 'dungeontile.png'.
		newBox x: i.
		newBox y: j.
		(i = 1) ifTrue: [newBox westWall: true].
		(j = 1) ifTrue: [newBox northWall: true].
		newBox width: spacing.
		newBox height: spacing.
		newBox position: ((i-1)*spacing)@((j-1)*spacing).

		boxMatrix at: i at: j put: newBox. 
	].

	^boxMatrix! !

!GameBoard methodsFor: 'mazing' stamp: 'WestonPanther 7/16/2014 15:24'!
mazify: boxMatrix withSet: setMatrix withWalls: wallList
		| tempBox |
		
		wallList do:  [ :wall |
		tempBox := wall at: 1.
		((wall at: 2) = true) ifTrue: [ 
			(tempBox x < boardSize) ifTrue: [ 
				((setMatrix at: (tempBox x) at: (tempBox y)) = (setMatrix at: (tempBox x+1) at: (tempBox y)) )  ifFalse: [ 
					
					tempBox eastWall: false.
					(setMatrix at: (tempBox x) at: (tempBox y)) do: [ :each | (setMatrix at: each x at: each y) addAll: (setMatrix at: (tempBox x + 1) at: (tempBox y)).].
					(setMatrix at: (tempBox x + 1) at: (tempBox y)) do: [ :each | setMatrix at: each x at: each y put: (setMatrix at: (tempBox x ) at: (tempBox y)).].
					
				]
			]
		] ifFalse: [ 
			(tempBox y < boardSize) ifTrue: [ 
				((setMatrix at: (tempBox x) at: (tempBox y)) = (setMatrix at: (tempBox x) at: (tempBox y + 1)) )  ifFalse: [ 
					
					tempBox southWall: false.
					(setMatrix at: (tempBox x) at: (tempBox y)) do: [ :each | (setMatrix at: each x at: each y) addAll: (setMatrix at: (tempBox x) at: (tempBox y + 1)).].
					(setMatrix at: (tempBox x) at: (tempBox y + 1)) do: [ :each | setMatrix at: each x at: each y put: (setMatrix at: (tempBox x ) at: (tempBox y)).].
				]
			]
		].
	].

	^boxMatrix! !

!GameBoard methodsFor: 'modifying' stamp: 'HenrySolberg 7/16/2014 01:13'!
moveGameObjectDown:  aGameObject
	(self wallAt: aGameObject x at: aGameObject y inDirection: 2) ifFalse:
	[ aGameObject moveDownOnBoard: self. ] .! !

!GameBoard methodsFor: 'modifying' stamp: 'HenrySolberg 7/16/2014 01:12'!
moveGameObjectLeft:  aGameObject
	(self wallAt: aGameObject x at: aGameObject y inDirection: 3) ifFalse:
	[aGameObject moveLeftOnBoard: self.]! !

!GameBoard methodsFor: 'modifying' stamp: 'HenrySolberg 7/16/2014 01:13'!
moveGameObjectRight:  aGameObject
	(self wallAt: aGameObject x at: aGameObject y inDirection: 1) ifFalse:
	[aGameObject moveRightOnBoard: self.]! !

!GameBoard methodsFor: 'modifying' stamp: 'HenrySolberg 7/16/2014 01:13'!
moveGameObjectUp:  aGameObject
	(self wallAt: aGameObject x at: aGameObject y inDirection: 0) ifFalse:
	[aGameObject moveUpOnBoard: self.]! !

!GameBoard methodsFor: 'accessing' stamp: 'WestonPanther 7/14/2014 16:44'!
player
	^player! !

!GameBoard methodsFor: 'modifying' stamp: 'WestonPanther 7/14/2014 16:39'!
player: aPlayer
	player := aPlayer.! !

!GameBoard methodsFor: 'accessing' stamp: 'HEnrySolberg 7/10/2014 04:37'!
positionOfX: anX Y: aY
	^(board at: anX at: aY) position.! !

!GameBoard methodsFor: 'modifying' stamp: 'HenrySolberg 7/17/2014 09:02'!
putBox: newBox spacing: spacing
	newBox resize: spacing@spacing.
	newBox drawBorders: 2.
	board at: (newBox x) at: (newBox y) put: newBox.
	newBox position: newBox position + self position.
	self addMorph: newBox.
	^newBox.
! !

!GameBoard methodsFor: 'accessing' stamp: 'HenrySolberg 7/16/2014 01:23'!
wallAt: x at: y inDirection: aDirection
	|thisBox northBox eastBox southBox westBox|
	thisBox := (self boxAt: x at: y).
	northBox := (self boxAt: x at: y-1).
	eastBox := (self boxAt: x+1 at: y).
	southBox := (self boxAt: x at: y+1).
	westBox := (self boxAt: x-1 at: y).
	(aDirection = 0) ifTrue: [ ^ thisBox northWall | northBox southWall ].
	(aDirection = 1) ifTrue: [ ^ thisBox eastWall | eastBox westWall ].
	(aDirection = 2) ifTrue: [ ^ thisBox southWall | southBox northWall ].
	(aDirection = 3) ifTrue: [ ^ thisBox westWall |westBox eastWall ].
	^self error: 'invalid direction'.! !

!GameBoard methodsFor: 'accessing' stamp: 'WestonPanther 7/16/2014 17:34'!
windowSize
	^windowSize! !

ImageMorph subclass: #GameBox
	instanceVariableNames: 'x y occupant usable northWall eastWall southWall westWall lineWidth'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MultiplayerDungeonGame'!

!GameBox commentStamp: '<historical>' prior: 0!
A GameBox is a box on the game grid.  has walls, denizen occupant, and usable item.

Instance Variables
	eastWall:		bool
	lineWidth:		bool
	northWall:		bool
	occupant:		denizen
	southWall:		bool
	usable:		usable
	westWall:		bool
	x:		int
	y:		int
!

!GameBox class methodsFor: 'as yet unclassified' stamp: 'HenrySolberg 7/17/2014 02:27'!
northWall: aBool1 eastWall: aBool2 southWall: aBool3 westWall: aBool4 tileImage: aString.
	^self new northWall: aBool1 eastWall: aBool2 southWall: aBool3 westWall: aBool4 tileImage: aString.! !

!GameBox methodsFor: 'hashing' stamp: 'HenrySolberg 7/15/2014 03:51'!
binaryWallRepresentation
	|representation|
	representation := 0.
	northWall ifTrue: [ representation := representation + 1000 ].
	eastWall ifTrue: [ representation := representation + 100 ].
	southWall ifTrue: [ representation := representation + 10 ].
	westWall ifTrue: [ representation := representation + 1 ].
	^Integer readFrom: representation asString base: 2.! !

!GameBox methodsFor: 'drawing' stamp: 'HenrySolberg 7/17/2014 02:27'!
drawBorders
	|topMorph rightMorph bottomMorph leftMorph|
	topMorph := Morph new.
	rightMorph := Morph new.
	bottomMorph := Morph new.
	leftMorph := Morph new.
	topMorph color: Color black.
	rightMorph color: Color black.
	bottomMorph color: Color black.
	leftMorph color: Color black.
	topMorph position: self left@self top.
	topMorph height: lineWidth.
	topMorph width: self width.
	rightMorph position: (self right - lineWidth) @ self top.
	rightMorph width: lineWidth.
	rightMorph height: self height.
	bottomMorph position: self left@(self bottom - lineWidth).
	bottomMorph height: lineWidth.
	bottomMorph width: self width.
	leftMorph position: self left@ self top.
	leftMorph width: lineWidth.
	leftMorph height: self height.
	northWall ifTrue: [self addMorph: topMorph].
	eastWall ifTrue: [self addMorph: rightMorph].
	southWall ifTrue: [self addMorph: bottomMorph].
	westWall ifTrue: [self addMorph: leftMorph].! !

!GameBox methodsFor: 'drawing' stamp: 'HenrySolberg 7/16/2014 00:24'!
drawBorders: lineWidth
	|topMorph rightMorph bottomMorph leftMorph|
	topMorph := Morph new.
	rightMorph := Morph new.
	bottomMorph := Morph new.
	leftMorph := Morph new.
	topMorph color: Color black.
	rightMorph color: Color black.
	bottomMorph color: Color black.
	leftMorph color: Color black.
	topMorph position: self left@self top.
	topMorph height: lineWidth.
	topMorph width: self width.
	rightMorph position: (self right - lineWidth) @ self top.
	rightMorph width: lineWidth.
	rightMorph height: self height.
	bottomMorph position: self left@(self bottom - lineWidth).
	bottomMorph height: lineWidth.
	bottomMorph width: self width.
	leftMorph position: self left@ self top.
	leftMorph width: lineWidth.
	leftMorph height: self height.
	northWall ifTrue: [self addMorph: topMorph].
	eastWall ifTrue: [self addMorph: rightMorph].
	southWall ifTrue: [self addMorph: bottomMorph].
	westWall ifTrue: [self addMorph: leftMorph].! !

!GameBox methodsFor: 'accessing' stamp: 'HenrySolberg 7/16/2014 01:18'!
eastWall
	^eastWall.! !

!GameBox methodsFor: 'modifying' stamp: 'WestonPanther 7/16/2014 13:29'!
eastWall: aBool
	eastWall := aBool.! !

!GameBox methodsFor: 'hashing' stamp: 'HenrySolberg 7/15/2014 03:52'!
hash
	^occupant hash * 100 + self binaryWallRepresentation.! !

!GameBox methodsFor: 'initializing' stamp: 'HenrySolberg 7/17/2014 02:27'!
initialize
	super initialize.
	x := 0.
	y := 0.
	northWall := true.
	eastWall := true.
	southWall := true.
	westWall := true.
	occupant := nil.
	lineWidth := 2.! !

!GameBox methodsFor: 'accessing' stamp: 'HenrySolberg 7/16/2014 01:18'!
northWall
	^northWall.! !

!GameBox methodsFor: 'modifying' stamp: 'WestonPanther 7/16/2014 13:29'!
northWall: aBool
	northWall := aBool.
! !

!GameBox methodsFor: 'initializing' stamp: 'HenrySolberg 7/17/2014 02:25'!
northWall: aBool1 eastWall: aBool2 southWall: aBool3 westWall: aBool4 tileImage: aString
	self form: (ImageReadWriter formFromFileNamed: aString).
	self northWall: aBool1.
	self eastWall: aBool2.
	self southWall: aBool3.
	self westWall: aBool4.! !

!GameBox methodsFor: 'accessing' stamp: 'HEnrySolberg 7/10/2014 05:23'!
occupant
	^occupant! !

!GameBox methodsFor: 'modifying' stamp: 'HEnrySolberg 7/10/2014 05:23'!
setOccupant: aGameObject.
	occupant := aGameObject.! !

!GameBox methodsFor: 'modifying' stamp: 'WestonPanther 7/16/2014 16:16'!
setUsable: aUsable
	usable := aUsable.! !

!GameBox methodsFor: 'accessing' stamp: 'HenrySolberg 7/16/2014 01:18'!
southWall
	^southWall.! !

!GameBox methodsFor: 'modifying' stamp: 'WestonPanther 7/16/2014 13:30'!
southWall: aBool
	southWall := aBool.
! !

!GameBox methodsFor: 'accessing' stamp: 'WestonPanther 7/16/2014 16:16'!
usable
	^usable! !

!GameBox methodsFor: 'accessing' stamp: 'HenrySolberg 7/16/2014 01:18'!
westWall
	^westWall.! !

!GameBox methodsFor: 'modifying' stamp: 'WestonPanther 7/16/2014 13:30'!
westWall: aBool
	westWall := aBool.
! !

!GameBox methodsFor: 'accessing' stamp: 'HenrySolberg 7/16/2014 04:27'!
x
	^x! !

!GameBox methodsFor: 'modifying' stamp: 'HEnrySolberg 7/10/2014 04:33'!
x: anX
	x := anX.! !

!GameBox methodsFor: 'accessing' stamp: 'HenrySolberg 7/16/2014 04:27'!
y
	^y! !

!GameBox methodsFor: 'modifying' stamp: 'HEnrySolberg 7/10/2014 04:24'!
y: aY
	y := aY.! !

ImageMorph subclass: #GameObject
	instanceVariableNames: 'x y'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MultiplayerDungeonGame'!

!GameObject commentStamp: '<historical>' prior: 0!
A GameObject represents an entity that can occupy a space in the dungeon.

Instance Variables
	x:		<Object>
	y:		<Object>

x
	- xxxxx

y
	- xxxxx
!

GameObject subclass: #Denizen
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MultiplayerDungeonGame'!

!Denizen commentStamp: '<historical>' prior: 0!
A Denizen is a living, breathing, moving, attacking game object.
!

!Denizen methodsFor: 'as yet unclassified' stamp: 'HenrySolberg 7/17/2014 05:15'!
isUsable
	^false! !

!Denizen methodsFor: 'as yet unclassified' stamp: 'WestonPanther 7/16/2014 16:28'!
use: anObject
	(anObject isUsable) ifTrue: [ 
		anObject applyTo: self.	
	]! !

!Denizen methodsFor: 'as yet unclassified' stamp: 'WestonPanther 7/16/2014 17:33'!
useOnBoard:aBoard
	|anObject|
	anObject:= (aBoard boxAt: x at: y) usable.
	(anObject = nil) ifFalse: [ 
		anObject applyTo: self onBoard: aBoard.
	]! !

!GameObject class methodsFor: 'as yet unclassified' stamp: 'WestonPanther 7/16/2014 15:57'!
imagePath: imagePath
	^self new imagePath: imagePath.! !

!GameObject methodsFor: 'using' stamp: 'WestonPanther 7/16/2014 16:28'!
applyTo: aPlayer
	self subclassResponsibility ! !

!GameObject methodsFor: 'hashing' stamp: 'HenrySolberg 7/16/2014 02:59'!
hash
	^(self class asString hash * 10000) + (x hash * 100 ) + (y hash * 1)! !

!GameObject methodsFor: 'initializing' stamp: 'WestonPanther 7/16/2014 15:58'!
imagePath: aString
	self form:(ImageReadWriter formFromFileNamed: aString).! !

!GameObject methodsFor: 'initializing' stamp: 'HenrySolberg 7/15/2014 04:01'!
initialize
	super initialize.
	self x: 1.
	self y: 1.! !

!GameObject methodsFor: 'using' stamp: 'WestonPanther 7/16/2014 16:18'!
isUsable
	^self subclassResponsibility ! !

!GameObject methodsFor: 'moving' stamp: 'HEnrySolberg 7/10/2014 05:50'!
moveDownOnBoard: aBoard
	self moveOnBoard: aBoard toX: x toY: y + 1.! !

!GameObject methodsFor: 'moving' stamp: 'HEnrySolberg 7/10/2014 05:50'!
moveLeftOnBoard: aBoard
	self moveOnBoard: aBoard toX: x - 1 toY: y.! !

!GameObject methodsFor: 'moving' stamp: 'HEnrySolberg 7/10/2014 05:51'!
moveOnBoard: aBoard toX: toX toY: toY
	aBoard fromX: x fromY: y toX: toX toY: toY.
	x := toX.
	y := toY.! !

!GameObject methodsFor: 'moving' stamp: 'HEnrySolberg 7/10/2014 05:48'!
moveRightOnBoard: aBoard
	self moveOnBoard: aBoard toX: x + 1 toY: y.
	! !

!GameObject methodsFor: 'moving' stamp: 'HEnrySolberg 7/10/2014 05:50'!
moveUpOnBoard: aBoard
	self moveOnBoard: aBoard toX: x toY: y - 1.! !

!GameObject methodsFor: 'accessing' stamp: 'HenrySolberg 7/16/2014 00:57'!
x
	^x.! !

!GameObject methodsFor: 'modifying' stamp: 'HEnrySolberg 7/10/2014 04:34'!
x: anX
	x := anX.! !

!GameObject methodsFor: 'accessing' stamp: 'HenrySolberg 7/16/2014 00:57'!
y
	^y.! !

!GameObject methodsFor: 'modifying' stamp: 'HEnrySolberg 7/10/2014 04:34'!
y: aY
	y := aY.! !

GameObject subclass: #Usable
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MultiplayerDungeonGame'!

!Usable commentStamp: '<historical>' prior: 0!
An Usable is something that can be used with the enter key to some effect.
!

Usable subclass: #Stairs
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MultiplayerDungeonGame'!

!Stairs commentStamp: '<historical>' prior: 0!
A Stairs is what gets you to the next level.
!

!Stairs methodsFor: 'as yet unclassified' stamp: 'HenrySolberg 7/17/2014 06:02'!
applyTo: aPlayer
	! !

!Stairs methodsFor: 'as yet unclassified' stamp: 'HenrySolberg 7/18/2014 04:22'!
applyTo: aPlayer onBoard: aBoard
	aBoard boardSize: aBoard boardSize windowSize: aBoard windowSize keepWindow: aBoard window.
	aBoard at: 1 at: 1 addGameObject: aPlayer.! !

!Stairs methodsFor: 'as yet unclassified' stamp: 'HenrySolberg 7/18/2014 04:18'!
applyTo: aPlayer onBoard: aBoard inWindow: aWindow
	aBoard boardSize: aBoard boardSize windowSize: aBoard windowSize keepWindow: aWindow.
	aBoard at: 1 at: 1 addGameObject: aPlayer.! !

!Usable methodsFor: 'as yet unclassified' stamp: 'WestonPanther 7/16/2014 16:18'!
isUsable
	^true! !
